<?php
/**
* @package RSFiles!
* @copyright (C) 2010-2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/
defined('_JEXEC') or die('Restricted access');

class rsfilesModelStatistics extends JModelList
{
	protected $_query;
	protected $_squery;
	protected $_data;
	protected $_sdata;
	protected $_total=null;
	protected $_stotal=null;
	protected $_pagination=null;
	protected $_spagination=null;
	protected $_layout;
	
	public function __construct() {	
		parent::__construct();
		$this->_buildQuery();
		
		$app = JFactory::getApplication();

		$this->_layout = $app->input->get('layout','default');
		
		// Get pagination request variables
		$limit 		= $app->getUserStateFromRequest('com_rsfiles.'.$this->_layout.'.limit', 'limit', $this->getState('list.limit'), 'int');
		$limitstart = $app->getUserStateFromRequest('com_rsfiles.'.$this->_layout.'.limitstart', 'limitstart', 0, 'int');

		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('com_rsfiles.'.$this->_layout.'.limit', $limit);
		$this->setState('com_rsfiles.'.$this->_layout.'.limitstart', $limitstart);
	}
	
	protected function populateState($ordering = null, $direction = null) {
		parent::populateState('s.path', 'desc');
    }
	
	protected function _buildQuery() {
		$db		= JFactory::getDbo();
		$filter = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$id		= JFactory::getApplication()->input->getInt('id',0);
		
		$query = $db->getQuery(true)->select($db->qn('f.IdFile'))->select($db->qn('f.FilePath'))->select($db->qn('f.hits'))->select('COUNT('.$db->qn('sd.id').') AS downloads')
					->select($db->qn('s.id'))->select($db->qn('s.path'))->select($db->qn('s.created_by'))->select($db->qn('u.name'))->select($db->qn('s.created_date'))->select($db->qn('s.removed_date'))
					->from($db->qn('#__rsfiles_statistics_files','s'))
					->join('LEFT',$db->qn('#__users','u').' ON '.$db->qn('u.id').' = '.$db->qn('s.created_by'))
					->join('LEFT',$db->qn('#__rsfiles_files','f').' ON '.$db->qn('f.IdFile').' = '.$db->qn('s.idf'))
					->join('LEFT',$db->qn('#__rsfiles_statistics_data','sd').' ON '.$db->qn('s.id').' = '.$db->qn('sd.ids'))
					->group($db->qn('s.path'))->group($db->qn('s.idf'))
					->order($db->qn('f.FilePath').','.$db->qn('s.path').' DESC');
				
		if ($filter) {
			$query->where('('.$db->qn('f.FilePath').' LIKE '.$db->q('%'.$filter.'%').' OR '.$db->qn('s.path').' LIKE '.$db->q('%'.$filter.'%').')');
		}
		
		$this->_query = (string) $query;
		
		$query = $db->getQuery(true)->select('s.*')->select($db->qn('u.username'))
					->from($db->qn('#__rsfiles_statistics_data','s'))
					->join('LEFT',$db->qn('#__users','u').' ON '.$db->qn('u.id').' = '.$db->qn('s.idu'))
					->where($db->qn('s.ids').' = '.$db->q($id))
					->order($db->qn('s.date').' DESC');
		
		$this->_squery = (string) $query;
	}
	
	public function getData() {
		if (empty($this->_data)) {
			$db	= JFactory::getDbo();
			$db->setQuery($this->_query,$this->getState('com_rsfiles.'.$this->_layout.'.limitstart'), $this->getState('com_rsfiles.'.$this->_layout.'.limit'));
			$this->_data = $db->loadObjectList();
		}
		
		return $this->_data;
	}
	
	public function getTotal() {
		if (empty($this->_total)) {
			$db	= JFactory::getDbo();
			$db->setQuery($this->_query);
			$db->execute();
			$this->_total = $db->getNumRows();
		}
		
		return $this->_total;
	}
	
	public function getPagination()	{
		if (empty($this->_pagination)) {
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('com_rsfiles.'.$this->_layout.'.limitstart'), $this->getState('com_rsfiles.'.$this->_layout.'.limit'));
		}
		return $this->_pagination;
	}
	
	public function getStatistics() {
		if (empty($this->_sdata)) {
			$db	= JFactory::getDbo();
			$db->setQuery($this->_squery,$this->getState('com_rsfiles.'.$this->_layout.'.limitstart'), $this->getState('com_rsfiles.'.$this->_layout.'.limit'));
			$this->_sdata = $db->loadObjectList();
		}

		return $this->_sdata;
	}
	
	public function getsTotal() {
		if (empty($this->_stotal)) {
			$db	= JFactory::getDbo();
			$db->setQuery($this->_squery);
			$db->execute();
			$this->_stotal = $db->getNumRows();
		}
		
		return $this->_stotal;
	}
 
	public function getsPagination() {
		if (empty($this->_spagination)) {
			jimport('joomla.html.pagination');
			$this->_spagination = new JPagination($this->getsTotal(), $this->getState('com_rsfiles.'.$this->_layout.'.limitstart'), $this->getState('com_rsfiles.'.$this->_layout.'.limit'));
		}
		return $this->_spagination;
	}
	
	public function getFilepath() {
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('f.FilePath'))->select($db->qn('s.path'))
			->from($db->qn('#__rsfiles_statistics_files','s'))
			->join('LEFT', $db->qn('#__rsfiles_files','f').' ON '.$db->qn('s.idf').' = '.$db->qn('f.IdFile'))
			->where($db->qn('s.id').' = '.JFactory::getApplication()->input->getInt('id',0));
		$db->setQuery($query);
		$path = $db->loadObject();
		
		return $path->FilePath ? $path->FilePath : $path->path;
	}
	
	public function clear($id) {
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()->delete($db->qn('#__rsfiles_statistics_data'))->where($db->qn('ids').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	public function delete($pks) {
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		if ($pks) {
			$pks = array_map('intval',$pks);
			
			$query->clear()->delete($db->qn('#__rsfiles_statistics_files'))->where($db->qn('id').' IN ('.implode(',',$pks).')');
			$db->setQuery($query);
			$db->execute();
			
			$query->clear()->delete($db->qn('#__rsfiles_statistics_data'))->where($db->qn('ids').' IN ('.implode(',',$pks).')');
			$db->setQuery($query);
			$db->execute();
		}
		
		return true;
	}
	
	public function csv($id) {
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$csv	= '';
		
		$query->clear()->select('s.*')->select($db->qn('u.username'))
			->from($db->qn('#__rsfiles_statistics_data','s'))
			->join('LEFT',$db->qn('#__users','u').' ON '.$db->qn('u.id').' = '.$db->qn('s.idu'))
			->where($db->qn('s.ids').' = '.$db->q($id))
			->order($db->qn('s.date').' DESC');
		$db->setQuery($query);
		if ($statistics = $db->loadObjectList()) {
			$query->clear()
				->select($db->qn('f.FilePath'))->select($db->qn('s.path'))
				->from($db->qn('#__rsfiles_statistics_files','s'))
				->join('LEFT', $db->qn('#__rsfiles_files','f').' ON '.$db->qn('s.idf').' = '.$db->qn('f.IdFile'))
				->where($db->qn('s.id').' = '.$id);
			$db->setQuery($query);
			$pathObj = $db->loadObject();
		
			$path		= $pathObj->FilePath ? $pathObj->FilePath : $pathObj->path;
			$filepath	= basename($path);
			
			header('Content-type: text/csv');
			header('Content-Disposition: attachment; filename="'.htmlentities(JText::sprintf('COM_RSFILES_VIEW_STATISTIC_FOR',$filepath), ENT_COMPAT, 'UTF-8').'.csv"');
			header('Pragma: no-cache');
			header('Expires: 0');
			
			foreach ($statistics as $row) {
				$csv .= "'".$db->escape($row->date)."','".$db->escape(($row->idu ? $row->username : JText::_('COM_RSFILES_GUEST')))."','".$db->escape($row->ip)."','".$db->escape($row->referer)."'\n";
			}
			
			echo $csv;
			die;
		} else {
			JFactory::getApplication()->enqueueMessage(JText::_('COM_RSFILES_NO_DATA'), 'error');
			JFactory::getApplication()->redirect(JRoute::_('index.php?option=com_rsfiles&view=statistics&layout=view&id='.$id, false));
		}
	}
}