<?php
/**
* @package RSFiles!
* @copyright (C) 2010-2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined('_JEXEC') or die('Restricted access');

if (file_exists(JPATH_SITE.'/components/com_rsfiles/helpers/rsfiles.php')) {
	require_once JPATH_SITE.'/components/com_rsfiles/helpers/rsfiles.php';
}

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.path');

class RSFilesAdminFiles
{
	/*
	*	The main folder
	*/
	public $folder = null;
	
	/*
	*	The RSFiles! configuration object
	*/
	private $config = null;
	
	/*
	*	The database object
	*/
	private $_db = null;
	
	/*
	*	Directory separator
	*/
	protected $ds;
	
	/*
	*	The order
	*/
	protected $order = null;
	
	/*
	*	The order direction
	*/
	protected $orderdir = null;
	
	/*
	*	Filter for files
	*/
	protected $filter = '.';
	
	/*
	*	Main constructor
	*/
	public function __construct($folder, $order = null, $orderdir = null) {
		// Set the database object
		$this->_db = JFactory::getDBO();
		
		// Set the current root folder
		$this->folder = $folder;
		
		// Set the configuration object
		$this->config = $this->getConfig();
		
		// Set directory separator
		$this->ds = rsfilesHelper::ds();
		
		// Set the default ordering
		$this->order = $order;
		
		// Set the default ordering direction
		$this->orderdir = strtoupper($orderdir);
	}
	
	/**
	 *	Modifies a property of the object, creating it if it does not already exist.
	 */
	public function set($name, $value) {
		$this->{$name} = $value;
	}
	
	/*
	*	Get the available files from the main folder
	*/
	public function getFiles() {
		$query			= $this->_db->getQuery(true);
		$input			= JFactory::getApplication()->input;
		$search 		= mb_strtolower($input->getString('filter_search',''));
		$return			= array();
		$files			= array();
		
		// Set the current download folder
		$root		= rsfilesHelper::getRoot();
		$briefcase	= $root == 'briefcase';
		$dld_fld	= $this->config->{$root.'_folder'}.$this->ds;
		
		// Search trough files
		if (!empty($search)) {
			if ($files_ftp = JFolder::files($this->config->{$root.'_folder'}, $this->filter, true, true, array('.htaccess'))) {
				foreach($files_ftp as $file) {
					if (mb_strpos(mb_strtolower(rsfilesHelper::getName($file)),$search) !== FALSE ) {
						if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
							$file = str_replace('/',"\\",$file);
						$files[] = $file;
					}
				}
			}
			
			$query->clear()
				->select($this->_db->qn('FilePath'))
				->from($this->_db->qn('#__rsfiles_files'))
				->where($this->_db->qn('briefcase').' = '.$this->_db->q((int) $briefcase))
				->where('( LOWER('.$this->_db->qn('FileName').') LIKE '.$this->_db->q('%'.$search.'%').' OR LOWER('.$this->_db->qn('FileDescription').') LIKE '.$this->_db->q('%'.$search.'%').')');
			
			$this->_db->setQuery($query);
			if ($search_files = $this->_db->loadColumn()) {
				foreach ($search_files as $search_file) {
					if (is_file($dld_fld.$search_file)) {
						$files[] = $dld_fld.$search_file;
					}
				}
			}
			
			$files = array_unique($files);
		} else {
			// Get files from this folder
			$files = JFolder::files($this->folder, $this->filter, false, true, array('.htaccess'));
		}
		
		// Parse files
		if (!empty($files)) {
			foreach ($files as $file) {
				if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
					$file = str_replace('/',"\\",$file);
				}
				
				$return[] = $this->getPath($file);
			}
			
			$return = $this->getInfo($return, 'file', $briefcase);
		}
		
		// Sort files
		switch($this->order) {
			default:
			case 'name':
				$return = rsfilesHelper::sort_array_name($return, $this->orderdir);
			break;
			
			case 'date':
				if ($this->orderdir == 'ASC')
					usort($return, array('rsfilesHelper', 'sort_time_asc'));
				else
					usort($return, array('rsfilesHelper', 'sort_time_desc'));
			break;
		}
		
		return $return;
	}
	
	/*
	*	Get the available folders from the main folder
	*/
	public function getFolders() {
		$query		= $this->_db->getQuery(true);
		$input		= JFactory::getApplication()->input;
		$search 	= mb_strtolower($input->getString('filter_search',''));
		$return		= array();
		$folders	= array();
		
		$root		= rsfilesHelper::getRoot();
		$briefcase	= $root == 'briefcase';
		$dld_fld	= $this->config->{$root.'_folder'}.$this->ds;
		
		if (!empty($search)) {
			if ($folders_ftp = JFolder::folders($this->config->{$root.'_folder'}, '.', true, true, array('.htaccess'))) {
				foreach($folders_ftp as $folder) {
					if (mb_strpos(mb_strtolower(rsfilesHelper::getName($folder)),$search) !== FALSE ) {
						if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
							$folder = str_replace('/',"\\",$folder);
						}
						$folders[] = $folder;
					}
				}
			}
			
			$query->clear()
				->select($this->_db->qn('FilePath'))
				->from($this->_db->qn('#__rsfiles_files'))
				->where($this->_db->qn('briefcase').' = '.$this->_db->q((int) $briefcase))
				->where('(LOWER('.$this->_db->qn('FileName').') LIKE '.$this->_db->q('%'.$search.'%').' OR LOWER('.$this->_db->qn('FileDescription').') LIKE '.$this->_db->q('%'.$search.'%').')');
			
			$this->_db->setQuery($query);
			if ($search_folders = $this->_db->loadColumn()) {
				foreach($search_folders as $search_folder) {
					if (is_dir($dld_fld.$search_folder)) {
						$folders[] = $dld_fld.$search_folder;
					}
				}
			}
			
			$folders = array_unique($folders);
		} else {
			$folders = JFolder::folders($this->folder, '.', false, true, array('.htaccess'));
		}
		
		if (!empty($folders)) {
			foreach ($folders as $folder) {
				if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
					$folder = str_replace('/',"\\",$folder);
				}
				
				$return[] = $this->getPath($folder);
			}
			
			$return = $this->getInfo($return, 'folder', $briefcase);
		}
		
		switch($this->order) {
			default:
			case 'name':
				$return = rsfilesHelper::sort_array_name($return, $this->orderdir);
			break;
			
			case 'date':
				if ($this->orderdir == 'ASC')
					usort($return, array('rsfilesHelper', 'sort_time_asc'));
				else
					usort($return, array('rsfilesHelper', 'sort_time_desc'));
			break;
		}
		
		return $return;
	}
	
	/*
	*	Get the external files
	*/
	public function getExternal() {
		$query		= $this->_db->getQuery(true);
		$search 	= mb_strtolower(JFactory::getApplication()->input->getString('filter_search',''));
		$return		= array();
		
		if ($this->folder == $this->config->download_folder) {
			$folder = 'root';
		} else {
			$folder = str_replace($this->config->download_folder,'',$this->folder);
			$folder = trim($folder,$this->ds);
		}
		
		if (!empty($search)) {
			$query->clear()
				->select('DISTINCT f.*')
				->from($this->_db->qn('#__rsfiles_files','f'))
				->where($this->_db->qn('f.FileType').' = 1');
		} else {
			$query->clear()
				->select('DISTINCT f.*')
				->from($this->_db->qn('#__rsfiles_files','f'))
				->where($this->_db->qn('f.FileType').' = 1')
				->where($this->_db->qn('f.FileParent').' = '.$this->_db->q($folder));
		}
		
		$query->join('LEFT',$this->_db->qn('#__rsfiles_licenses','l').' ON '.$this->_db->qn('l.IdLicense').' = '.$this->_db->qn('f.IdLicense'));
		
		$this->_db->setQuery($query);
		$externals = $this->_db->loadObjectList();
		
		if (!empty($externals)) {
			foreach ($externals as $external) {				
				if (!empty($search)) {
					if (mb_strpos(mb_strtolower(rsfilesHelper::getName($external->FileName)),$search) === FALSE && mb_strpos(mb_strtolower(rsfilesHelper::getName($external->FilePath)),$search) === FALSE)
						continue;
				}
				
				$object = new stdClass();
				
				$object->id					= !empty($external->IdFile) ? $external->IdFile : 0;
				$object->name				= !empty($external->DownloadName) ? $external->DownloadName : rsfilesHelper::getName($external->FilePath);
				$object->filename			= !empty($external->FileName) ? $external->FileName : '';
				$object->fullpath			= (int) $external->IdFile;
				$object->time				= !empty($external->DateAdded) ? $external->DateAdded : JFactory::getDate()->toSql();
				$object->type				= 'external';
				$object->dateadded 			= rsfilesHelper::showDate($object->time);
				$object->published			= isset($external->published) ? $external->published : 1;
				$object->fileversion 		= !empty($external->FileVersion) ? $external->FileVersion : '';
				$object->Downloads 			= !empty($external->Downloads) ? $external->Downloads : 0;
				$object->DownloadLimit 		= !empty($external->DownloadLimit) ? $external->DownloadLimit : '';
				$object->filelicense 		= empty($external->IdLicense) ? '' : '<a href="'.JRoute::_('index.php?option=com_rsfiles&task=license.edit&IdLicense='.$external->IdLicense,false).'">'.$external->LicenseName.'</a>';
				$object->hits				= empty($external->hits) ? 0 : (int) $external->hits;
				$object->stats				= isset($external->FileStatistics) ? $external->FileStatistics : '';
				
				$return[] = $object;
			}
			
			switch($this->order) {
				default:
				case 'name':
					$return = rsfilesHelper::sort_array_name($return, $this->orderdir);
				break;
				
				case 'date':
					if ($this->orderdir == 'ASC')
						usort($return, array('rsfilesHelper', 'sort_time_asc'));
					else
						usort($return, array('rsfilesHelper', 'sort_time_desc'));
				break;
			}
		}
		
		return $return;
	}
	
	/*
	*	Get the navigation
	*/
	public static function getPathNavigation($path) {
		$navigation 	= ''; 
		$config 		= self::getConfig();
		$ds				= rsfilesHelper::ds();
		$layout			= JFactory::getApplication()->input->get('layout','');
		$modal			= $layout == 'modal' ? '&layout=modal&tmpl=component' : '';
		$from			= JFactory::getApplication()->input->get('from','');
		$from			= !empty($from) ? '&from='.$from : '';
		$editor			= JFactory::getApplication()->input->getString('editor','') ? '&editor='.JFactory::getApplication()->input->getString('editor','') : '';
		
		if(!empty($config->briefcase_folder) && $config->briefcase_folder == substr($path, 0, strlen($config->briefcase_folder))) {
			$root 			= 'briefcase_folder';
			$navigation 	= '<a href="'.JRoute::_('index.php?option=com_rsfiles&view=files'.$from.$editor.$modal).'">'.str_replace($ds,' '.$ds.' ',$config->briefcase_folder).'</a> '.$ds;
			$path  			= str_replace($config->briefcase_folder,'',$path);
		} else {
			$root 			= 'download_folder';
			$navigation 	= '<a href="'.JRoute::_('index.php?option=com_rsfiles&view=files'.$from.$editor.$modal).'">'.str_replace($ds,' '.$ds.' ',$config->download_folder).'</a> '.$ds;
			$path  			= str_replace($config->download_folder,'',$path );
		}
		
		if (substr($path,0,1) == $ds) {
			$path = ltrim($path,$ds);
		}
		
		if (!empty($path)) {
			$folders	 = explode($ds, $path);
			$total		 = count($folders);
			$folder_path = '';
			
			$i = 0;
			foreach($folders as $folder) {
				if ($i != 0 && $i != $total)
					$folder_path .= $ds;
				
				$folder_path 	.= $folder;
				
				$navigation 	.= ' <a href="'.JRoute::_('index.php?option=com_rsfiles&view=files'.$from.$editor.$modal.'&folder='.urlencode($folder_path)).'">'.$folder.'</a> '.$ds;
				$i++;
			}
		}
		
		return $navigation;
	}
	
	/*
	*	Set files filter
	*/
	public function setFilter($filter = '.') {
		$this->filter = $filter;
	}
	
	/*
	*	Get the configuration
	*/
	protected static function getConfig() {
		return rsfilesHelper::getConfig();
	}
	
	protected function getInfo($paths, $type, $briefcase) {
		$return 	= array();
		$root 		= $this->getRoot();
		$query 		= $this->_db->getQuery(true);
		$newpaths	= $this->quoteImplode($paths);
		$briefcase	= $briefcase ? 1 : 0;
		
		$query->select($this->_db->qn('f').'.*')->select($this->_db->qn('l.LicenseName'))->from($this->_db->qn('#__rsfiles_files','f'))->where($this->_db->qn('f.FilePath').' IN ('.implode(',', $newpaths).')');
		$query->join('LEFT',$this->_db->qn('#__rsfiles_licenses','l').' ON '.$this->_db->qn('l.IdLicense').' = '.$this->_db->qn('f.IdLicense'));
		$query->where($this->_db->qn('briefcase').' = '.$this->_db->q($briefcase));
		$this->_db->setQuery($query);
		$data = $this->_db->loadObjectList('FilePath');
		
		foreach ($paths as $path) {
			$object = new stdClass();
			$file	= isset($data[$path]) ? $data[$path] : false;
			
			$fullpath					= realpath($root.$this->ds.$path);
			$object->id					= !empty($file->IdFile) ? $file->IdFile : 0;
			$object->name				= rsfilesHelper::getName($fullpath);
			$object->filename			= !empty($file->FileName) ? $file->FileName : '';
			$object->fullpath			= isset($file->FilePath) ? urlencode($file->FilePath) : urlencode($path);
			$object->time				= !empty($file->DateAdded) && $file->DateAdded != $this->_db->getNullDate() ? $file->DateAdded : JFactory::getDate(filemtime($fullpath))->toSql();
			$object->type				= $type;
			$object->dateadded 			= rsfilesHelper::showDate($object->time);
			$object->published			= isset($file->published) ? $file->published : 1;
			$object->fileversion 		= !empty($file->FileVersion) ? $file->FileVersion : '';
			$object->Downloads 			= !empty($file->Downloads) ? $file->Downloads : 0;
			$object->DownloadLimit 		= !empty($file->DownloadLimit) ? $file->DownloadLimit : '';
			$object->filelicense 		= empty($file->IdLicense) ? '' : '<a href="'.JRoute::_('index.php?option=com_rsfiles&task=license.edit&IdLicense='.$file->IdLicense,false).'">'.$file->LicenseName.'</a>';
			$object->hits				= empty($file->hits) ? 0 : (int) $file->hits;
			$object->stats				= isset($file->FileStatistics) ? $file->FileStatistics : '';
			$object->reports			= $this->getReport($object->id);
			
			$return[] = $object;
		}
		
		return $return;
	}
	
	/**
	 *	Get the number of reports a file has
	 */
	protected function getReport($id) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
			
		$query->select('COUNT('.$db->qn('IdReport').') AS ReportsCount')
			->from($db->qn('#__rsfiles_reports'))
			->where($db->qn('IdFile').' = '.$db->q($id));
			
		$db->setQuery($query);
		$report = (int) $db->loadResult();
		
		if ($report <= 0) {
			return JText::_('COM_RSFILES_NO_REPORT');
		} elseif ($report == 1) {
			return JText::_('COM_RSFILES_ONE_REPORT');
		} else {
			return JText::sprintf('COM_RSFILES_MANY_REPORT',$report);
		}
	}
	
	protected function getPath($fullpath) {
		$root	= $this->getRoot();
		$path	= str_replace($root,'',$fullpath);
		
		return ltrim($path,rsfilesHelper::ds());
	}
	
	protected function getRoot() {
		$briefcase 	= rsfilesHelper::getRoot() == 'briefcase';
		$d_root		= realpath($this->config->download_folder);
		$b_root		= realpath($this->config->briefcase_folder);
		
		return $briefcase ? $b_root : $d_root;
	}
	
	protected function quoteImplode($paths) {
		foreach ($paths as $i => $path) {
			$paths[$i] = $this->_db->q($path);
		}
		
		return $paths;
	}
}